package com.trust.ywx.customer.service.query.impl;

import com.fiftyonetrust.common.redis.template.RedisRepository;
import com.trust.ywx.customer.common.enu.FirmProperEnum;
import com.trust.ywx.customer.common.enu.NetworkTypeEnum;
import com.trust.ywx.customer.common.model.PageBean;
import com.trust.ywx.customer.common.model.PageBeanUtil;
import com.trust.ywx.customer.dao.*;
import com.trust.ywx.customer.dao.po.FirmInfo;
import com.trust.ywx.customer.dao.po.FirmInfoExtend;
import com.trust.ywx.customer.dao.po.FirmInfoTemp;
import com.trust.ywx.customer.dao.po.FirmUsercertChannel;
import com.trust.ywx.customer.common.model.manager.response.TrustFirmDto;
import com.trust.ywx.customer.common.model.query.request.SearchCrmFirmParam;
import com.trust.ywx.customer.common.model.query.request.SearchFirmParam;
import com.trust.ywx.customer.common.model.query.request.SearchTrustFirmParam;
import com.trust.ywx.customer.common.model.manager.response.TempFirmDto;
import com.trust.ywx.customer.common.model.query.response.CrmFirmDto;
import com.trust.ywx.customer.service.query.FirmQueryService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;

/**
 * <pre></pre>
 *
 * @文件名称： FirmQueryServiceImpl.java
 * @包 路  径： com.trust.ywx.customer.service.query.impl
 * @Copyright：北京数字医信责任有限公司 (C) 2022 *
 * @Description:
 * @Version: V1.0
 * @Author： hanzhenchao
 * @Date：2022/6/15 18:43
 * @Modify：
 */
@CacheConfig(cacheNames = {"firm"})
@Service("firmQueryService")
public class FirmQueryServiceImpl implements FirmQueryService {
    @Autowired
    private TrustFirmDao trustFirmDao;
    @Autowired
    private TempFirmDao tempFirmDao;
    @Autowired
    private FirmGroupDao firmGroupDao;
    @Autowired
    private FirmExtendDao firmExtendDao;
    @Autowired
    private DoctorProductTypeDao doctorProductTypeDao;
    @Autowired
    RedisRepository redisRepository;
    @Override
    public PageBean<TrustFirmDto> getTrustFirmList(SearchTrustFirmParam param) {
        List<String> clientIdList = new ArrayList<>();

        // clientId筛选
        if (StringUtils.isNotEmpty(param.getClientId())) {
            FirmInfo firmInfo = trustFirmDao.getFrimByClientId(param.getClientId());
            if (null == firmInfo) {
                return PageBeanUtil.getEmptyPage(param.getPageNo(),param.getPageSize());
            }
            // 增加项目状态的判断
            if (null != param.getProjectStatus() && !Objects.equals(firmInfo.getProjectStatus(), param.getProjectStatus())) {
                return PageBeanUtil.getEmptyPage(param.getPageNo(),param.getPageSize());
            }
            // 增加实施人员的判断
            if (null != param.getTechId() && !Objects.equals(firmInfo.getTechPersonId(), param.getTechId())) {
                return PageBeanUtil.getEmptyPage(param.getPageNo(),param.getPageSize());
            }
            // 增加销售人员的判断
            if (null != param.getSaleId() && !Objects.equals(firmInfo.getSalePersonId(), param.getTechId())) {
                return PageBeanUtil.getEmptyPage(param.getPageNo(),param.getPageSize());
            }

            clientIdList.add(param.getClientId());
        }

        // 客户组筛选
        if (StringUtils.isNotEmpty(param.getGroupId())) {
            List<String> groupClientIdList = firmGroupDao.getClientIdByGroupId(param.getGroupId());
            if (CollectionUtils.isEmpty(clientIdList)) {
                clientIdList = groupClientIdList;
            } else {
                clientIdList.retainAll(groupClientIdList);
                if (CollectionUtils.isEmpty(clientIdList)) {
                    return PageBeanUtil.getEmptyPage(param.getPageNo(),param.getPageSize());
                }
            }
        }

        // 医师产品形态筛选 fixme 下面的查询可以合成一个
        if (null != param.getDoctorProductType()) {
            List<String> subClientIdList;
            if (StringUtils.isNotEmpty(param.getClientId())) {
                subClientIdList = getClientIdListByProp(FirmProperEnum.DOCTOR_PRODUCT_APP.getProperName(), param.getDoctorProductType(), Arrays.asList(param.getClientId()));
            } else {
                subClientIdList = getClientIdListByProp(FirmProperEnum.DOCTOR_PRODUCT_APP.getProperName(), param.getDoctorProductType(), null);
            }
            // 与clientIdList求并集
            clientIdList.retainAll(subClientIdList);
            if (CollectionUtils.isEmpty(clientIdList)) {
                return PageBeanUtil.getEmptyPage(param.getPageNo(),param.getPageSize());
            }

            if (StringUtils.isNotEmpty(param.getClientId())) {
                subClientIdList = getClientIdListByProp(FirmProperEnum.DOCTOR_PRODUCT_SDK.getProperName(), param.getDoctorProductType(), Arrays.asList(param.getClientId()));
            } else {
                subClientIdList = getClientIdListByProp(FirmProperEnum.DOCTOR_PRODUCT_SDK.getProperName(), param.getDoctorProductType(), null);
            }
            // 与clientIdList求并集
            clientIdList.retainAll(subClientIdList);
            if (CollectionUtils.isEmpty(clientIdList)) {
                return PageBeanUtil.getEmptyPage(param.getPageNo(),param.getPageSize());
            }
        }

        /**
         * 属性过滤暂时没想到好办法，只能一个个去查询了
         */
        if (null != param.getNetworkType()) {
            List<String> subClientIdList = getClientIdListByProp(FirmProperEnum.NETWORK_TYPE.getProperName(), param.getNetworkType(), clientIdList);
            // 与clientIdList求并集
            clientIdList.retainAll(subClientIdList);
            if (CollectionUtils.isEmpty(clientIdList)) {
                return PageBeanUtil.getEmptyPage(param.getPageNo(),param.getPageSize());
            }
        }
        if (null != param.getChargingMode()) {
            List<String> subClientIdList = getClientIdListByProp(FirmProperEnum.CHARGING_MODE.getProperName(), param.getChargingMode(), clientIdList);
            // 与clientIdList求并集
            clientIdList.retainAll(subClientIdList);
            if (CollectionUtils.isEmpty(clientIdList)) {
                return PageBeanUtil.getEmptyPage(param.getPageNo(),param.getPageSize());
            }
        }
        if (null != param.getPublicProductType()) {
            List<String> subClientIdList = getClientIdListByProp(FirmProperEnum.PUBLIC_PRODUCT_TYPE.getProperName(), param.getPublicProductType(), clientIdList);
            // 与clientIdList求并集
            clientIdList.retainAll(subClientIdList);
            if (CollectionUtils.isEmpty(clientIdList)) {
                return PageBeanUtil.getEmptyPage(param.getPageNo(),param.getPageSize());
            }
        }
        PageBean<FirmInfo> page = null;

        if(CollectionUtils.isEmpty(clientIdList)){
            page = trustFirmDao.getList(param.getPageNo(),param.getPageSize());
        }else{
            if (clientIdList.size() < param.getPageSize() * param.getPageNo()) {
                return PageBeanUtil.getEmptyPage(param.getPageNo(),param.getPageSize());
            }
            page = trustFirmDao.getListByIds(clientIdList,  param.getPageNo(),param.getPageSize());
        }

        if (CollectionUtils.isEmpty(page.getList())) {
            return PageBeanUtil.getEmptyPage(param.getPageNo(),param.getPageSize());
        }
        List<TrustFirmDto> dtoList = new ArrayList<>(page.getList().size());
        for (FirmInfo po : page.getList()) {
            TrustFirmDto dto = converPoToDto(po);
            dtoList.add(dto);
        }

        PageBean pageBean = new PageBean(dtoList);
        pageBean.setPageNum(page.getPageNum());
        pageBean.setPageSize(page.getPageSize());
        pageBean.setTotal(page.getTotal());
        pageBean.setPages(page.getPages());
        return pageBean;
    }


    private TrustFirmDto converPoToDto(FirmInfo po) {
        if (null == po) {
            return null;
        }
        TrustFirmDto dto = new TrustFirmDto();
        BeanUtils.copyProperties(po, dto);

        // TODO 差异属性转换
        return dto;
    }

    @Override
    public TrustFirmDto getTrustFirm(String clientId) {
        FirmInfo po = trustFirmDao.getFrimByClientId(clientId);
        return converPoToDto(po);
    }

    @Override
    public TrustFirmDto getTrustFirmWithBLOBs(String clientId) {
        FirmInfo po = trustFirmDao.getFrimByClientId(clientId);
        return converPoToDto(po);
    }

    @Override
    public Map<String, String> getAllProp(String clientId) {
        if (StringUtils.isEmpty(clientId)) {
            return new HashMap<>();
        }
        Map<String, String> map = new HashMap<>(64);
        // 1、从firm_info_extend获取全量属性
        List<FirmInfoExtend> extendList = firmExtendDao.getListByClientId(clientId);
        for (FirmInfoExtend extendPo : extendList) {
            map.put(extendPo.getPropName(), extendPo.getPropValue());
        }

        //2、从firm_usercert_channel 获取医师产品形态属性 fixme 以后要去掉
        List<FirmUsercertChannel> doctorProductList = doctorProductTypeDao.getByClientId(clientId);

        for (FirmUsercertChannel doctorPo : doctorProductList) {
            if (doctorPo.getChannelType() == 0) {
                map.put(FirmProperEnum.DOCTOR_PRODUCT_APP.getProperName(), doctorPo.getChannelType() + "");
            }
            if (doctorPo.getChannelType() == 1) {
                map.put(FirmProperEnum.DOCTOR_PRODUCT_SDK.getProperName(), doctorPo.getChannelType() + "");
            }
        }
        return map;
    }

    @Override
    public List<TrustFirmDto> getFirmWhite(String firmName) {
        return null;
    }

    @Override
    public PageBean<TempFirmDto> getTempFirmList(SearchFirmParam param) {
        return tempFirmDao.getListByParam(param);
    }

    @Override
    @Cacheable(key = "'temp:'+#clientId")
    public TempFirmDto getTempFirmByClientId(String clientId) {
        FirmInfoTemp tempPo = tempFirmDao.getByClientId(clientId);
        TempFirmDto dto =converTempPoToDto(tempPo);
        return dto;
    }

    private TempFirmDto converTempPoToDto(FirmInfoTemp tempPo) {
        TempFirmDto dto = new TempFirmDto();
        BeanUtils.copyProperties(tempPo,dto);
        if(StringUtils.isNotBlank(tempPo.getChannel())){
            if("0".equals(tempPo.getChannel())){
                dto.setChannelApp(true);
            }else if("1".equals(tempPo.getChannel())){
                dto.setChannelSdk(true);
            }else if("2".equals(tempPo.getChannel())){
                dto.setChannelApp(true);
                dto.setChannelSdk(true);
            }
        }
        return dto;
    }

    @Override
    public List<CrmFirmDto> getCrmFirmList(SearchCrmFirmParam param) {
        return null;
    }

    private List<String> getClientIdListByProp(String propName, Integer propValue, List<String> clientIdList) {
        String convertPropValue = "";
        if (Objects.equals(propName, FirmProperEnum.DOCTOR_PRODUCT_SDK.getProperName())
                || Objects.equals(propName, FirmProperEnum.DOCTOR_PRODUCT_APP.getProperName())) {
            //fixme，目前医师产品形态单独存储在firm_usercert_channel表，后期挪到firm_info_extends表
            return doctorProductTypeDao.getClientIdByProp(clientIdList, propValue);
        } else {
            // 其他属性都在firm_info_extends表
            if (Objects.equals(propName, FirmProperEnum.NETWORK_TYPE.getProperName())) {
                convertPropValue = NetworkTypeEnum.getNameOfCode(propValue);
            } else {
                convertPropValue = String.valueOf(propValue);
            }
            return firmExtendDao.getByPropsAndClientId(propName, convertPropValue, clientIdList);
        }
    }

}
